/**==========================================================================
Hochschule Luzern - Technik & Architektur            Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer, Mike Estermann
-----------------------------------------------------------------------------
Name        : Uebung_2-3.c
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
              Mike Estermann <mike.estermann@stud.hslu.ch>
Version     : 2012-09-23v1.0
Description : Entwurf bung mit Vektoren und malloc / realloc
===========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <time.h>

/**
 * Memory Allozieren
 * @param size Anzahl MB die alliziert werden sollen
 * @return int-Vector
 */
int* allocate(long size);

/**
 * Sensless-Methode um den Vector zu benutzen.
 * Random Nummern speichern und wieder lesen.
 * @param vector Pointer auf den Vector
 */
void useVector(int *vector);


/**
 * The main procedure to run the application.
 * @param argc Number of command line arguments
 * @param argv The forwarded command line arguments
 * @return Application return (error) code
 */
int main(int argc, char** argv)
{
	// Variablen definieren
	int wert;
	long size;
	int *vector;
	
	// get size
	printf("Wie viel MB soll alloziert werden: ");
	scanf("%d", &wert);
	size = wert * 1024 * 1024; // Eingaben in MB umrechnen
	fflush(stdin); //flushes the input stream and gets rid of '\n'
	
	// allocate vector
	vector = allocate(size);

	// use vector
	printf("\n[Enter] druecken um den Speicher zu benutzen...");
	getchar();
	useVector(vector);

	// free memory
	printf("\n[Enter] druecken um den Speicher freizugeben...");
	getchar();
	free(vector);

	// exit
	printf("\n\nPress [Enter] to exit...");
	getchar();
	return(EXIT_SUCCESS);
}


/**
 * Memory Allozieren
 * @param size Anzahl MB die alliziert werden sollen
 * @return int-Vector
 */
int* allocate(long size)
{
	// Variablen definieren
	const int steps = 20;
	int index = 0;
	int *vector;
	   
	// calc size based on integer-size
	size = size / sizeof(int);

	// Allocate the memory
	for (; index < steps; index++)
	{
		// Allocate or reallocate memory
		vector = (int *)((index == 0) ? malloc(sizeof(int) * size / steps) : realloc(vector, (sizeof(int) * size / steps) * index));
                           
		if (vector == NULL) 
		{
			// Error
			perror("\nNicht genug Speicher vorhanden!\n\n");
			// Exit
			exit(EXIT_FAILURE);
		}

		// delay for demo
		Sleep(500);
		printf("Memory allocated: %d\n", ((sizeof(int) * size * (index + 1)) / steps));
	}

	// return pointer of the new vector
	return vector;
}

/**
 * Sensless-Methode um den Vector zu benutzen.
 * Random Nummern speichern und wieder lesen.
 * @param vector Pointer auf den Vector
 */
void useVector(int *vector)
{
	// Variablen definieren
	int index;
	int countNumbers = 10;

	// Initialize random generator
	srand(time(NULL));

	// generate random numbers
	for (index = 0; index < countNumbers; index++)
	{
		*(vector + index) = rand();
	}

	// print numbers
	for(index = 0; index < countNumbers; index++)
	{
		printf("%i. %i\n", index + 1, *(vector + index));
	}
}